/*

  Copyright 2018 ZeroEx Intl.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

*/

pragma solidity ^0.5.5;

import "../interfaces/IAssets.sol";


contract MAssets is
    IAssets
{
    /// @dev Transfers given amount of asset to sender.
    /// @param assetData Byte array encoded for the respective asset proxy.
    /// @param amount Amount of asset to transfer to sender.
    function transferAssetToSender(
        bytes memory assetData,
        uint256 amount
    )
        internal;

    /// @dev Decodes ERC20 assetData and transfers given amount to sender.
    /// @param assetData Byte array encoded for the respective asset proxy.
    /// @param amount Amount of asset to transfer to sender.
    function transferERC20Token(
        bytes memory assetData,
        uint256 amount
    )
        internal;

    /// @dev Decodes ERC721 assetData and transfers given amount to sender.
    /// @param assetData Byte array encoded for the respective asset proxy.
    /// @param amount Amount of asset to transfer to sender.
    function transferERC721Token(
        bytes memory assetData,
        uint256 amount
    )
        internal;

    /// @dev Sets approval for ERC20 AssetProxy.
    /// @param assetData Byte array encoded for the respective asset proxy.
    /// @param amount Amount of asset to approve for respective proxy.
    function approveERC20Token(
        bytes memory assetData,
        uint256 amount
    )
        internal;

    /// @dev Sets approval for ERC721 AssetProxy.
    /// @param assetData Byte array encoded for the respective asset proxy.
    /// @param amount Amount of asset to approve for respective proxy.
    function approveERC721Token(
        bytes memory assetData,
        uint256 amount
    )
        internal;
}
